package ittp.protocol.parser;

import ittp.ittpException;
import ittp.protocol.response.Response;
import java.io.StreamTokenizer;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * The aim of this class is to scan the request send by the client.
 * @author Frederic Bidon and Mathieu Texier
 */
public class ITTPScan extends TokenStream {
    
    private StreamTokenizer strtok;
    
    /** Creates a new instance of Parser */
    public ITTPScan (BufferedReader stream) {
        strtok = new StreamTokenizer (stream);
        setup ();
        next ();
    }
    
    public void setup () {
        strtok.resetSyntax ();
        strtok.whitespaceChars (' ',  ' ');
        strtok.whitespaceChars ('\t',  '\t');
        strtok.parseNumbers ();
        strtok.wordChars ('a', 'z');
        strtok.wordChars ('A', 'Z');
        strtok.wordChars ('_', '_');
        strtok.wordChars ('.', '.');
        strtok.wordChars ('$', '$');
    }
    
    /**Method allowing to filter the different characters of a string.*/
    public void filterString () {
        strtok.resetSyntax ();
        strtok.whitespaceChars (' ',  ' ');
        strtok.whitespaceChars ('\t',  '\t');
        strtok.wordChars ('a', 'z');
        strtok.wordChars ('A', 'Z');
        strtok.wordChars ('0', '9');
        strtok.wordChars ('_', '_');
        strtok.wordChars ('.', '.');
        strtok.wordChars ('$', '$');
    }
    
    /**
     * This method analyse and identify the headers contained in the request.
     * @return the number associate to the headers.
     */
    public int next () {
        if (tok != EOF)
            try {
                tok = strtok.nextToken (); // block
                switch (tok) {
                    case StreamTokenizer.TT_EOF :
                        tok = EOF;
                        break;
                    case StreamTokenizer.TT_EOL:
                        tok = NL;
                        break;
                    case StreamTokenizer.TT_NUMBER:
                        tok = NUMBER;
                        nval = strtok.nval;
                        break;
                    case StreamTokenizer.TT_WORD:
                        sval = strtok.sval;
                        if (sval.equals ("GET"))                     tok = GET;
                        else if (sval.equals ("HEAD"))               tok = HEAD;
                        else if (sval.equals ("OPTIONS"))            tok = OPTIONS;
                        else if (sval.equals ("TRACE"))              tok = TRACE;
                        else if (sval.equals ("ittp"))               tok = ITTP;
                        else if (sval.equals ("Connection"))         tok = CONNECTION;
                        else if (sval.equals ("Date"))               tok = DATE;
                        else if (sval.equals ("Host"))               tok = HOST;
                        else if (sval.equals ("If-Modified-Since"))  tok = IF_MODIFIED_SINCE;
                        else if (sval.equals ("Range"))              tok = RANGE;
                        else tok = WORD;
                        break;
                    case '/': case '\\' :
                        tok = SLASH;
                        break;
                    case ':':
                        tok = COLON;
                        break;
                    case ',':
                        tok = COMA;
                        break;
                    case '-':
                        tok = MINUS;
                        break;
                    default :
                        throw new TokenException ("Illegal token " + this);
                }
            } catch (IOException e) {throw new TokenException (e.getMessage ());}
        return tok;
    }
    
    /**
     * This method determine the different parameters of the request that are
     * separated by a syntaxic character.
     * @param num int containing the number of token.
     * @return a string containing the next token of the request.
     */
    public String next (int num) {
        strtok.resetSyntax ();
        strtok.whitespaceChars (' ',  ' ');
        strtok.wordChars ('!', 'z');
        
        String retStr = strtok.sval;
        for (int i=0; i< num; i++)
            try {
                switch (tok = strtok.nextToken ()) {
                    
                    case StreamTokenizer.TT_WORD:
                        retStr += strtok.sval + " ";
                        break;
                    case '/': case '\\' : case ':':  case ',':
                    case '-': case '_': case '.': case '\'': case '$':
                        retStr += (char) tok;
                        break;
                    default :
                        throw new TokenException ("Illegal token " + this +" " + i);
                }
            } catch (IOException e) {
                setup ();
                throw new TokenException (e.getMessage ());
            }
        setup ();
        return retStr;
    }
    
    /**
     * Convert to a string the headers identified.
     * @return the headers converted in a string.
     */
    public String toString () {
        switch (tok) {
            case StreamTokenizer.TT_EOF: case EOF:
                return "<EOF>";
            default:
                return "" +(char) tok;
        }
    }
    
    /**
     * This class that extends the class ittpException send an exception when it is invoked.
     */
    class TokenException extends ittpException {
        /**
         * Constructor of the class.
         * @param s String containing the error message.
         */
        public TokenException (String s) {
            super (s, Response.Code.ITTP_BAD_REQUEST);
        }
    }
}
